#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <dlfcn.h>
#include <stdarg.h>

#include "../src/mlp_priv.h"
#include "mlp_c_priv.h"


void *_mlp_c_load_library(mlp_context *ctx, const char *name, const void *opts){
	void *handle = NULL ;
	char *buf = NULL ;

	if (name != NULL){
		buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
		sprintf(buf, "%s.so", name) ;
	}

	dlerror() ;
	/* TODO: Pass opts to dlopen */
	handle = dlopen(buf, RTLD_LAZY) ;
	if (handle == NULL){
		mlp_context_set_error(ctx, MLP_LOAD_LIBRARY_ERROR, "Can't load shared object %s: %s\n", 
			(buf != NULL ? buf : "(MAIN PROGRAM)"), dlerror()) ;
		free(buf) ;
		return NULL ;
	}

	mlp_c_library *library = (mlp_c_library *)malloc(sizeof(mlp_c_library)) ;
    library->name = (buf != NULL ? strdup(buf) : "(MAIN PROGRAM)") ;
    library->handle = handle ;
	mlp_debug(2, "Library %s loaded", name) ;
	if (buf != NULL){
		free(buf) ;
	}

	return library ;
}


mlp_value *_mlp_c_call_function(mlp_context *ctx, void *library, const char *name, 
	int nb_args, const mlp_value **args, mlp_type return_type){

	mlp_value *ret = NULL ;
	mlp_value *(*f)(int nb_args, const mlp_value **args) = NULL ;
	dlerror() ;
	f = dlsym(((mlp_c_library *)library)->handle, name) ;
	char *error ;
	if ((error = dlerror()) != NULL){
		mlp_context_set_error(ctx, MLP_NO_SUCH_FUNCTION, "Can't find symbol %s in library %s: %s\n", 
			name, ((mlp_c_library *)library)->name, error) ;
	}
	else {
		mlp_debug(3, "Function %s found in library %s loaded", name, ((mlp_c_library *)library)->name) ;
		ret = (*f)(nb_args, args) ;
		mlp_debug(3, "Function %s called:", name) ;
		int i ;
		for (i = 0 ; i < nb_args ; i++){
			mlp_debug_value(args[i], "arg[%d]", i) ;
		}
		mlp_debug_value(ret, "ret") ;
	}

	return ret ;
}
