#include "test.h"

void t_00_info(){
#line 1 "00_info.t"
	
	plan_tests(1) ;
	
	ok(1, "Test language is %s (%s)", "c", "") ;
}

void t_01_context(){
#line 1 "01_context.t"
	plan_tests(10) ;
	
	mlp_context ctx ;
	ok(! ctx.has_error(), "Context doesn't have a pending error") ;
	ok(ctx.get_error() == MLP_OK, "Context error is MLP_OK") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.set_error(MLP_INTERNAL_ERROR, NULL) ;
	ok(ctx.get_error() == MLP_INTERNAL_ERROR, "Context error is MLP_INTERNAL_ERROR") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.clear_error() ;
	ok(ctx.get_error() == MLP_OK, "Context error is MLP_OK") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.set_error(MLP_INTERNAL_ERROR, "internal %s", "error") ;
	ok(ctx.get_error() == MLP_INTERNAL_ERROR, "Context error is MLP_INTERNAL_ERROR") ;
	ok(strcmp(ctx.get_error_msg(), "internal error") == 0, "Context error msg is \"internal error\"") ;
	
	mlp_runtime *rt = NULL ;
	ctx.set_runtime_data(rt, (const void *)"data") ;
	ok(strcmp((const char  *)ctx.get_runtime_data(rt), "data") == 0, "Context runtime data for NULL is \"data\"") ;
	
}

void t_02_primitives(){
#line 1 "02_primitives.t"
	plan_tests(18) ;
	
	{
	mlp_value i(42) ;
	ok(i.to_int() == 42, "Int value as long is 42") ;
	ok(i.to_double() == (double)42, "Int value as double is 42.0") ;
	ok(strcmp(i.to_string(), "42") == 0, "Int value as string is \"42\"") ;
	}
	
	{
	mlp_value l(42L) ;
	ok(l.to_long() == 42L, "Long value as long is 42") ;
	ok(l.to_double() == (double)42L, "Long value as double is 42.0") ;
	ok(strcmp(l.to_string(), "42") == 0, "Long value as string is \"42\"") ;
	}
	
	{
	mlp_value d(42.6) ;
	ok(d.to_double() == 42.6, "Double value as double is 42.6") ;
	ok(d.to_long() == 42L, "Double value as long is 42") ;
	ok(strncmp(d.to_string(), "42.6", 4) == 0, "Double value as string is \"42.6\"", d.to_string()) ;
	}
	
	{
	mlp_value s("42.6") ;
	ok(strcmp(s.to_string(), "42.6") == 0, "String value as string is \"42.6\"") ;
	ok(s.to_long() == 42L, "String value as long is 42") ;
	ok(s.to_double() == 42.6, "String value as double is 42.6") ;
	}
	
	
	{
	mlp_value s("some string") ;
	ok(strcmp(s.to_string(), "some string") == 0, "String value as string is \"some string\"") ;
	ok(s.to_long() == 0L, "String value as long is 0") ;
	ok(s.to_double() == (double)0L, "String value as double is 0.0") ;
	}
	
	
	{
	mlp_value s((const char *)NULL) ;
	ok(s.to_string() == NULL, "String value as string is NULL") ;
	ok(s.to_long() == 0L, "String value as long is 0") ;
	// Here we test NANness by using x != x
	ok(s.to_double() != s.to_double(), "String value as double is NAN") ;
	}
}

void t_03_language(){
#line 1 "03_language.t"
	plan_tests(2) ;
	
	mlp_context ctx ;
	
	mlp_language *lang = mlp_language::init(&ctx, "c") ; // "" ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", "c") ;
	}
	
	
	lang->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", "c") ;
	}
}

void t_04_runtime(){
#line 1 "04_runtime.t"
	plan_tests(5) ;
	
	mlp_context ctx ;
	
	mlp_language *lang = mlp_language::init(&ctx, "c") ; // "" ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", "c") ;
	}
	mlp_runtime *rt = lang->new_runtime(&ctx, NULL) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Runtime for %s created", "c") ;
	}
	rt->setup_context(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Context set up for %s runtime", "c") ;
	}
	
	
	rt->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Runtime for %s destroyed", "c") ;
	}
	lang->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", "c") ;
	}
}

void t_05_library(){
#line 1 "05_library.t"
	plan_tests(7) ;
	
	mlp_context ctx ;
	
	mlp_language *lang = mlp_language::init(&ctx, "c") ; // "" ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", "c") ;
	}
	mlp_runtime *rt = lang->new_runtime(&ctx, NULL) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Runtime for %s created", "c") ;
	}
	rt->setup_context(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Context set up for %s runtime", "c") ;
	}
	
	mlp_library *lib = rt->load_library(&ctx, "calc", NULL) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lib != NULL, "Library calc loaded") ;
	}
	
	
	lib->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Library calc destroyed") ;
	}
	rt->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Runtime for %s destroyed", "c") ;
	}
	lang->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", "c") ;
	}
}

void t_06_function(){
#line 1 "06_function.t"
	plan_tests(20) ;
	
	mlp_context ctx ;
	
	mlp_language *lang = mlp_language::init(&ctx, "c") ; // "" ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", "c") ;
	}
	mlp_runtime *rt = lang->new_runtime(&ctx, NULL) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Runtime for %s created", "c") ;
	}
	rt->setup_context(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Context set up for %s runtime", "c") ;
	}
	mlp_library *lib = rt->load_library(&ctx, "calc", NULL) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lib != NULL, "Library calc loaded") ;
	}
	
	
	// LONG
	mlp_function *f = NULL ;
	{
	mlp_prototype proto ;
	proto.push_back(MLP_LONG) ;
	proto.push_back(MLP_LONG) ;
	f = lib->get_function(&ctx, MLP_LONG, "calc_add_long", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(f != NULL, "Function calc_add_long loaded") ;
	}
	}
	{
	mlp_arguments args ;
	args.push_back(mlp_value(2L)) ;
	args.push_back(mlp_value(3L)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
	ok(ret != NULL, "Function calc_add_long called with args (2L, 3L)") ;
	ok(ret->to_long() == 5L, "Result is 5L") ;
	delete ret ;
	}
	}
	
	// DOUBLE
	{
	mlp_prototype proto ;
	proto.push_back(MLP_DOUBLE) ;
	proto.push_back(MLP_DOUBLE) ;
	f = lib->get_function(&ctx, MLP_DOUBLE, "calc_add_double", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(f != NULL, "Function calc_add_double loaded") ;
	}
	}
	{
	mlp_arguments args ;
	args.push_back(mlp_value(2.0)) ;
	args.push_back(mlp_value("3.0")) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
	ok(ret != NULL, "Function calc_add_long called with args (2.0, 3.0)") ;
	ok(ret->to_double() == 5.0, "Result is 5.0") ;
	delete ret ;
	}
	}
	f->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Function calc_add_double destroyed") ;
	}
	
	// STRING
	{
	mlp_prototype proto ;
	proto.push_back(MLP_STRING) ;
	proto.push_back(MLP_STRING) ;
	f = lib->get_function(&ctx, MLP_STRING, "calc_add_string", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(f != NULL, "Function calc_add_string loaded") ;
	}
	}
	{
	mlp_arguments args ;
	args.push_back(mlp_value("2")) ;
	args.push_back(mlp_value(3L)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
	ok(ret != NULL, "Function calc_add_string called with args (\"2\", \"3\")") ;
	ok(strtod(ret->to_string(), NULL) == 5.0, "Result is \"5\"") ;
	delete ret ;
	}
	}
	{
	mlp_arguments args ;
	args.push_back(mlp_value((const char *)NULL)) ;
	args.push_back(mlp_value((const char *)NULL)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
	ok(ret != NULL, "Function calc_add_string called with args (NULL, NULL)") ;
	ok(ret->to_string() == NULL, "Result is NULL %s", ret->to_string()) ;
	delete ret ;
	}
	}
	f->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Function calc_add_string destroyed") ;
	}
	
	
	lib->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Library calc destroyed") ;
	}
	rt->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Runtime for %s destroyed", "c") ;
	}
	lang->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", "c") ;
	}
}

int main(int argc, char *argv[]){
	if (argc < 2) { fprintf(stderr, "Usage: test FILE\n") ; exit(1) ; }
	if (strcmp(argv[argc-1], "00_info.t") == 0) { t_00_info() ; }
	if (strcmp(argv[argc-1], "01_context.t") == 0) { t_01_context() ; }
	if (strcmp(argv[argc-1], "02_primitives.t") == 0) { t_02_primitives() ; }
	if (strcmp(argv[argc-1], "03_language.t") == 0) { t_03_language() ; }
	if (strcmp(argv[argc-1], "04_runtime.t") == 0) { t_04_runtime() ; }
	if (strcmp(argv[argc-1], "05_library.t") == 0) { t_05_library() ; }
	if (strcmp(argv[argc-1], "06_function.t") == 0) { t_06_function() ; }
	return exit_status() ;
}
