#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <dlfcn.h>
#include "mlp_c.h"


void *mlp_c_library_new(mlp_context *ctx, mlp_runtime *rt, const char *name, const void *opts){
	char *buf = NULL ;
	if (name != NULL){
		buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
		sprintf(buf, "lib%s.so", name) ;
	}

	CInvContext *cic = (CInvContext *)mlp_context_get_runtime_data(ctx, rt) ;
	CInvLibrary *cil = cinv_library_create(cic, buf) ;
	if (cil == NULL){
		mlp_context_set_error(ctx, MLP_LIBRARY_ERROR, "Can't load shared object %s: %s\n",
			(buf != NULL ? buf : "(MAIN PROGRAM)"), cinv_context_geterrormsg(cic)) ;
		free(buf) ;
		return NULL ;
	}
	mlp_debug(2, "C library %s loaded", buf) ;
	free(buf) ;

	return cil ;
}


void mlp_c_library_delete(mlp_context *ctx, mlp_library *lib){
	CInvContext *cic = (CInvContext *)mlp_context_get_runtime_data(ctx, lib->runtime) ;
	CInvLibrary *cil = (CInvLibrary *)lib->local ;
	cinv_status_t st = cinv_library_delete(cic, cil) ;
	if (st == CINV_ERROR){
		mlp_context_set_error(ctx, MLP_LIBRARY_ERROR, "Can't unload shared object %s: %s\n",
			lib->name, cinv_context_geterrormsg(cic)) ;
	}
}


/*
static void *mlp_c_library::_load_entrypoint(mlp_context *ctx, const char *name){
	ctx->clear_error() ;

	CInvContext *cic = (CInvContext *)ctx->get_runtime_data(rt) ;
	void *fptr = cinv_library_load_entrypoint(cic, cil, name) ;
	if (fptr == NULL){
		ctx->set_error(MLP_FUNCTION_ERROR, "Can't locate entrypoint '%s' in library '%s': %s\n",
			name, this->name.c_str(), cinv_context_geterrormsg(cic)) ;
		return NULL ;
	}

	return fptr ;
}
*/


