#include <stdlib.h>		/* for malloc */
#include <stdio.h>		/* for snprintf */
#include <stdarg.h>
#include "mlp_priv.h"


MLP_PUBLIC_API mlp_context *mlp_context_new(){
	mlp_context *ctx = (mlp_context *)malloc(sizeof(mlp_context)) ;
	ctx->err = MLP_OK ;
	ctx->err_msg = NULL ;
	int i ;
	for (i = 0 ; i < MLP_MAX_RUNTIMES ; i++){
		ctx->runtime_data[i] = NULL ;
	}

	return ctx ;
}


MLP_PUBLIC_API void mlp_context_delete(mlp_context *ctx){
	mlp_context_clear_error(ctx) ;
	free(ctx) ;
}


MLP_PUBLIC_API mlp_error mlp_context_has_error(mlp_context *ctx){
	return (ctx->err == MLP_OK ? 0 : 1) ;
}


MLP_PUBLIC_API mlp_error mlp_context_get_error(mlp_context *ctx){
	return ctx->err ;
}


/*
	Returns the error message associated with an mlp_context object, or NULL
	if there is no error message.
	Note: The returned char * is kept in an internal buffer and therefore must not be freed by the caller.
*/
MLP_PUBLIC_API const char *mlp_context_get_error_msg(mlp_context *ctx){
	return ctx->err_msg ;
}


MLP_PUBLIC_API void mlp_context_set_error(mlp_context *ctx, mlp_error e, const char *fmt, ...){
	mlp_context_clear_error(ctx) ;
	ctx->err = e ;

	if (fmt != NULL){
		va_list va ;
		va_start(va, fmt) ;
		char *msg = (char *)malloc(MLP_MAX_ERROR_MSG_LENGTH * sizeof(char)) ;
		vsnprintf(msg, MLP_MAX_ERROR_MSG_LENGTH - 1, fmt, va) ;
		ctx->err_msg = msg ;
		va_end(va) ;
	}
}


/*
	Clears the error for an mlp_context object. This implies freeing the error message
	and resetting the error status to MLP_OK.
*/
MLP_PUBLIC_API void mlp_context_clear_error(mlp_context *ctx){
	ctx->err = MLP_OK ;
	if (ctx->err_msg != NULL){
		free(ctx->err_msg) ;
		ctx->err_msg = NULL ;
	}
}


/*
	This function is used by mlp_runtime_init_context() to associate language specific data
	to the context.
*/
MLP_PRIVATE_API void mlp_context_set_runtime_data(mlp_context *ctx, mlp_runtime *rt, const void *data){
	int rt_id = mlp_runtime_get_id(rt) ;
	ctx->runtime_data[rt_id] = data ;
}


/*
	This function is used by mlp_runtime_uninit_context() to remove language specific data
	from the context.
*/
MLP_PRIVATE_API void *mlp_context_get_runtime_data(mlp_context *ctx, mlp_runtime *rt){
	int rt_id = mlp_runtime_get_id(rt) ;
	return (void *)ctx->runtime_data[rt_id] ;
}
