use strict ;

my $mode = shift @ARGV ;
die("Invalid mode '$mode'.\n") unless (($mode eq 'public')||($mode eq 'private')) ;

my %API = {} ;
foreach my $file (@ARGV){
	open(F, "<$file") or die("Can't open '$file' for reading: $!\n") ;
	while (<F>){	
		my $line = $_ ;
		chomp($line) ;
		$line =~ s/\s*{\s*$// ;
		my ($public, $private, $local) = () ;
		if ($line =~ s/\bMLP_PUBLIC_API\b//){
			$public = 1 ;
		}
		if ($line =~ s/\bMLP_PRIVATE_API\b//){
			$private = 1 ;
		}
		if ($line =~ s/\bMLP_LOCAL_API\b//){
			$local = 1 ;
		}
		
		$line =~ s/^\s+// ;
		push @{$API{$file}->{'public'}}, $line if $public ;
		push @{$API{$file}->{'private'}}, $line if $private ;
		push @{$API{$file}->{'local'}}, $line if $local ;
	}	
	close(F) ;
}

if ($mode eq 'public'){
	print "/* mlp_p.h, generated on ", scalar(localtime()), " */\n" ;
	print_prototypes() ;
}
else {
	print "/* mlp_priv_p.h, generated on ", scalar(localtime()), " */\n" ;
	print_prototypes() ;
	print_local_api() ;
}


sub print_prototypes {
	foreach my $f (@ARGV){
		my @lines = @{$API{$f}->{$mode} || []} ;
		if (scalar(@lines)){
			print "\n/* From $f: */\n" ;
			foreach my $line (@lines){
				print "$line ;\n" ;
			}
		}
	}
	print "\n\n" ;
}


sub print_local_api {
	print "struct _mlp_local_api {\n" ;
	foreach my $f (@ARGV){
		my @lines = @{$API{$f}->{'local'} || []} ;
		if (scalar(@lines)){
			print "\n\t/* From $f: */\n" ;
			foreach my $line (@lines){
				if ($line =~ /^(.*?)mlp_([a-z]+_.*?)\s*\(\s*(.*?)\s*\)/){
					my $ret = $1 ;
					my $func = $2 ;
					my @proto = split(/\s*,\s*/, $3) ;
					$ret = "void *" if ($ret =~ /\*/) ;
					print "\t$ret(*$func)(" . join(', ', @proto) . ") ;\n" ;
				}
			}
		}
	}
	print "} ;\n" ;
	print "typedef struct _mlp_local_api mlp_local_api ;\n" ;

	print "\n\n" ;
	print "#define MLP_GENERATE_LOCAL_API(local, lang) \\\n" ;
	print "\t(mlp_local_api *)malloc(sizeof(mlp_local_api)) ; \\\n" ;
	foreach my $f (@ARGV){
		my @lines = @{$API{$f}->{'local'} || []} ;
		if (scalar(@lines)){
			print "\t\\\n\t/* From $f: */ \\\n" ;
			foreach my $line (@lines){
				if ($line =~ /^(.*?)mlp_([a-z]+_.*?)\s*\(\s*(.*?)\s*\)/){
					my $ret = $1 ;
					my $func = $2 ;
					my @proto = split(/\s*,\s*/, $3) ;
					$ret = "void *" if ($ret =~ /\*/) ;
					print "\t${ret}mlp_ ## lang ## _$func(" . join(', ', @proto) . ") ; \\\n" ;
					print "\tlocal->$func = mlp_ ## lang ## _$func ; \\\n" ;
				}
			}
		}
	}
	print "\n\n" ;
}



__DATA__

#define MLP_GENERATE_INIT(iface, lang) \
	(mlp_runtime_interface *)malloc(sizeof(mlp_runtime_interface)) ; \
	/**/ \
	void *_mlp_ ## lang ## _new_runtime(mlp_context *ctx, const void *opts) ; \
	iface->new_runtime = _mlp_ ## lang ## _new_runtime ; \
	/**/ \
	void _mlp_ ## lang ## _delete_runtime(mlp_context *ctx, void *runtime) ; \
	iface->delete_runtime = _mlp_ ## lang ## _delete_runtime ; \
	/**/ \
	void *_mlp_ ## lang ## _load_library(mlp_context *ctx, const char *name, const void *opts) ; \
	iface->load_library = _mlp_ ## lang ## _load_library ; \
	/**/ \
	mlp_value *_mlp_## lang ## _convert_value(mlp_context *ctx, mlp_value *value, mlp_type type) ; \
	iface->convert_value = _mlp_ ## lang ## _convert_value ; \
	/**/ \
	mlp_value *_mlp_## lang ## _call_function(mlp_context *ctx, void *library, const char *name, \
		int nb_args, const mlp_value **args, mlp_type return_type) ; \
	iface->call_function = _mlp_ ## lang ## _call_function ; \
	/**/ \
	void *_mlp_## lang ## _new_class(mlp_context *ctx, const char *name) ; \
	iface->new_class = _mlp_ ## lang ## _new_class ; \
	void *_mlp_## lang ## _new_object(mlp_context *ctx, const mlp_class *class_, \
	    int nb_args, const mlp_value **args) ; \
	iface->new_object = _mlp_ ## lang ## _new_object ; \
	mlp_value *_mlp_## lang ## _call_method(mlp_context *ctx, const mlp_object *object, const char *name, \
		int nb_args, const mlp_value **args, mlp_type return_type) ; \
	iface->call_method = _mlp_ ## lang ## _call_method ; \
	void _mlp_## lang ## _detach_object(mlp_context *ctx, const mlp_object *object) ; \
	iface->detach_object = _mlp_ ## lang ## _detach_object ; \
	void _mlp_## lang ## _delete_object(mlp_context *ctx, const mlp_object *object) ; \
	iface->delete_object = _mlp_ ## lang ## _delete_object ; \

#endif

