#include <stdlib.h> 		/* for malloc */
#include <string.h>			/* for strlen */
#include <stdio.h>			/* for sprintf */
#include <dlfcn.h>			/* for dlopen, dlsym */
#include "mlp_priv.h"


MLP_PUBLIC_API mlp_language *mlp_language_init(mlp_context *ctx, const char *name){
	mlp_context_clear_error(ctx) ;
	dlerror() ; 

	char *buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
	sprintf(buf, "libmlp_%s.so", name) ;
	void *handle = dlopen(buf, RTLD_LAZY) ;
	if (handle == NULL){
		mlp_context_set_error(ctx, MLP_PLUGIN_ERROR, "Can't open shared object %s: %s\n", buf, dlerror()) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;

	/* Now that the plugin is loaded, we must initialize it */
	buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
	sprintf(buf, "mlp_%s_language_init", name) ;

	dlerror() ; 
	mlp_local_api *(*init)(const mlp_context *ctx) = NULL ;
	init = dlsym(handle, buf) ;
	const char *error ;
	if ((error = dlerror()) != NULL){
		mlp_context_set_error(ctx, MLP_PLUGIN_ERROR, "Can't find symbol %s: %s\n", buf, error) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;

	mlp_local_api *api = (*init)(ctx) ;
	if (mlp_context_has_error(ctx)){
		return NULL ;
	}

	mlp_language *lang = (mlp_language *)malloc(sizeof(mlp_language)) ;
	lang->name = strdup(name) ;
	lang->handle = handle ;
	lang->api = api ;

	return lang ;
}


MLP_PUBLIC_API void mlp_language_delete(mlp_context *ctx, mlp_language *lang){
	mlp_context_clear_error(ctx) ;

	if (lang->handle != NULL){
		int rc = dlclose(lang->handle) ;
		if (rc != 0){
			mlp_context_set_error(ctx, MLP_PLUGIN_ERROR, "Can't close shared object %s: %s\n", lang->name, dlerror()) ;
		}
	}
	if (lang->api != NULL){
		free(lang->api) ;
	}
	free(lang->name) ;
	free(lang) ;
}


MLP_PUBLIC_API const char *mlp_language_get_name(mlp_context *ctx, mlp_language *lang){
	mlp_context_clear_error(ctx) ;

	return lang->name ;
}
