#include <stdlib.h>
#include "mlp_priv.h"


static int next_id = 1 ;


MLP_PUBLIC_API MLP_LOCAL_API mlp_runtime *mlp_runtime_new(mlp_context *ctx, mlp_language *lang, const void *opts){
	mlp_context_clear_error(ctx) ;

	void *local_runtime = (*(lang->api->runtime_new))(ctx, lang, opts) ;
	if (mlp_context_has_error(ctx)){
		return NULL ;
	}

	mlp_runtime *runtime = (mlp_runtime *)malloc(sizeof(mlp_runtime)) ;
	runtime->language = lang ;
	runtime->id = next_id++ ;
	runtime->local = local_runtime ;

	return runtime ;
}


MLP_PUBLIC_API MLP_LOCAL_API void mlp_runtime_delete(mlp_context *ctx, mlp_runtime *rt){
	mlp_context_clear_error(ctx) ;

    (*(rt->language->api->runtime_delete))(ctx, rt) ;
	if (mlp_context_has_error(ctx)){
		return ;
	}

	free(rt) ;
}


MLP_PRIVATE_API int mlp_runtime_get_id(mlp_runtime *rt){
	/* Here we allow NULL as a special case for the test suite. */
	if (rt == NULL){
		return 0 ;
	}

	return rt->id ;
}


MLP_PUBLIC_API MLP_LOCAL_API void mlp_runtime_init_context(mlp_context *ctx, mlp_runtime *rt){
	mlp_context_clear_error(ctx) ;

    (*(rt->language->api->runtime_init_context))(ctx, rt) ;
}

