plan_tests(20) ;

mlp_context ctx ;

mlp_language *lang = mlp_language::init(&ctx, MLP_TEST_LANG) ; // MLP_TEST_OPTS ;
MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", MLP_TEST_LANG) ;
}
mlp_runtime *rt = lang->new_runtime(&ctx, NULL) ;
MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Runtime for %s created", MLP_TEST_LANG) ;
}
rt->setup_context(\&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	ok(rt != NULL, "Context set up for %s runtime", MLP_TEST_LANG) ;
}
mlp_library *lib = rt->load_library(&ctx, "calc", NULL) ;
MLP_CHECK_ERROR(ctx, 1){
	ok(lib != NULL, "Library calc loaded") ;
}


// LONG
mlp_function *f = NULL ;
{
	mlp_prototype proto ;
	proto.push_back(MLP_LONG) ;
	proto.push_back(MLP_LONG) ;
	f = lib->get_function(&ctx, MLP_LONG, "calc_add_long", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
		ok(f != NULL, "Function calc_add_long loaded") ;
	}
}
{
	mlp_arguments args ;
	args.push_back(mlp_value(2L)) ;
	args.push_back(mlp_value(3L)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
		ok(ret != NULL, "Function calc_add_long called with args (2L, 3L)") ;
		ok(ret->to_long() == 5L, "Result is 5L") ;
		delete ret ;
	}
}

// DOUBLE
{
	mlp_prototype proto ;
	proto.push_back(MLP_DOUBLE) ;
	proto.push_back(MLP_DOUBLE) ;
	f = lib->get_function(&ctx, MLP_DOUBLE, "calc_add_double", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
		ok(f != NULL, "Function calc_add_double loaded") ;
	}
}
{
	mlp_arguments args ;
	args.push_back(mlp_value(2.0)) ;
	args.push_back(mlp_value("3.0")) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
		ok(ret != NULL, "Function calc_add_long called with args (2.0, 3.0)") ;
		ok(ret->to_double() == 5.0, "Result is 5.0") ;
		delete ret ;
	}
}
f->destroy(&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	pass("Function calc_add_double destroyed") ;
}

// STRING
{
	mlp_prototype proto ;
	proto.push_back(MLP_STRING) ;
	proto.push_back(MLP_STRING) ;
	f = lib->get_function(&ctx, MLP_STRING, "calc_add_string", proto) ;
	MLP_CHECK_ERROR(ctx, 1){
		ok(f != NULL, "Function calc_add_string loaded") ;
	}
}
{
	mlp_arguments args ;
	args.push_back(mlp_value("2")) ;
	args.push_back(mlp_value(3L)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
		ok(ret != NULL, "Function calc_add_string called with args (\\"2\\", \\"3\\")") ;
		ok(strtod(ret->to_string(), NULL) == 5.0, "Result is \\"5\\"") ;
		delete ret ;
	}
}
{
	mlp_arguments args ;
	args.push_back(mlp_value((const char *)NULL)) ;
	args.push_back(mlp_value((const char *)NULL)) ;
	mlp_value *ret = f->call(&ctx, args) ;
	MLP_CHECK_ERROR(ctx, 2){
		ok(ret != NULL, "Function calc_add_string called with args (NULL, NULL)") ;
		ok(ret->to_string() == NULL, "Result is NULL %s", ret->to_string()) ;
		delete ret ;
	}
}
f->destroy(&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	pass("Function calc_add_string destroyed") ;
}


lib->destroy(&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	pass("Library calc destroyed") ;
}
rt->destroy(&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	pass("Runtime for %s destroyed", MLP_TEST_LANG) ;
}
lang->destroy(&ctx) ;
MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", MLP_TEST_LANG) ;
}
