#include "mlp_impl.h"
#include <dlfcn.h>


// static map to hold all the language objects
std::map<std::string, mlp_language_impl *> mlp_language_impl::languages ;


mlp_language_impl::mlp_language_impl(const char *l) : name(l), handle(NULL){
	languages[name] = this ;
}


mlp_language_impl::~mlp_language_impl(){
}


const char *mlp_language_impl::get_name(mlp_context *ctx){
	return name.c_str() ;
}


bool mlp_language_impl::destroy(mlp_context *ctx){
	languages[name] = NULL ;
	if (handle != NULL){
		std::string name = this->name ;
		int rc = dlclose(handle) ;
		if (rc != 0){
			ctx->set_error(MLP_PLUGIN_ERROR, "Can't close shared object %s: %s\n", name.c_str(), dlerror()) ;
			return false ;
		}
	}
	return true ;
}


mlp_language_impl *mlp_language_impl::init(mlp_context *ctx, const char *name){
	ctx->clear_error() ;
	mlp_language_impl *l = languages[std::string(name)] ;
	if (l != NULL){
		// Language already initialized.
		return l ;
	}

	dlerror() ; 
	std::string buf = std::string("libmlp_") + name + ".so" ;
	void *handle = dlopen(buf.c_str(), RTLD_LAZY | RTLD_GLOBAL) ;
	if (handle == NULL){
		ctx->set_error(MLP_PLUGIN_ERROR, "Can't open shared object %s: %s\n", buf.c_str(), dlerror()) ;
		return NULL ;
	}

	// The fact of loading the .so should have created and registered the language object.
	// We can now get it from the map and return it.
	l = languages[std::string(name)] ;
	if (l == NULL){
		ctx->set_error(MLP_PLUGIN_ERROR, "Language plugin %s (%s) did not register a language object", 
			name, buf.c_str()) ;
		return NULL ;
	}
	l->handle = handle ;

	return l ;
}
