#include "mlp_api.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


static mlp_value *mlp_value_new(mlp_type type){
    mlp_value *value = MLP_ALLOC(mlp_value) ;
    value->type = type ;
    value->thrown = 0 ;
    value->stringy = NULL ;

    return value ;
}


extern "C" mlp_value *mlp_value_from_int(int val){	
	mlp_value *value = mlp_value_new(MLP_INT) ;
	value->v.i = val ;
	return value ;
}


extern "C" mlp_value *mlp_value_from_long(long val){
	mlp_value *value = mlp_value_new(MLP_LONG) ;
	value->v.l = val ;
	return value ;
}


extern "C" mlp_value *mlp_value_from_double(double val){
	mlp_value *value = mlp_value_new(MLP_DOUBLE) ;
	value->v.d = val ;
	return value ;
}


extern "C" mlp_value *mlp_value_from_string(const char *val){
	mlp_value *value = mlp_value_new(MLP_STRING) ;
	value->v.s = (val == NULL ? NULL : strdup(val)) ;
	return value ;
}


extern "C" mlp_value *mlp_value_from_array(const mlp_array *val){
	mlp_value *value = mlp_value_new(MLP_ARRAY) ;
	value->v.a = val ;
	return value ;
}


extern "C" mlp_value *mlp_value_from_object(const mlp_object *val){
	mlp_value *value = mlp_value_new(MLP_ARRAY) ;
	value->v.o = val ;
	return value ;
}


extern "C" void mlp_value_delete(mlp_value *val){
	switch (val->type){
		case MLP_STRING:
			if (val->v.s != NULL){
				free(val->v.s) ;
			}
			break ;
		}

	if (val->stringy != NULL){
		free(val->stringy) ;
	}
	MLP_FREE(val) ;
}


extern "C" int mlp_value_to_int(const mlp_value *val){
	return (int) mlp_value_to_long(val) ;
}


extern "C" long mlp_value_to_long(const mlp_value *val){
	switch (val->type){
		case MLP_INT:
			return (long)val->v.i ;
		case MLP_LONG:
			return val->v.l ;
		case MLP_DOUBLE:
			return (long)val->v.d ;
		case MLP_STRING:
			return (val->v.s == NULL ? 0L : strtol(val->v.s, NULL, 10)) ;
		default:
			return 0L ;
	}
}


extern "C" double mlp_value_to_double(const mlp_value *val){
	switch (val->type){
		case MLP_INT:
			return (double)val->v.i ;
		case MLP_LONG:
			return (double)val->v.l ;
		case MLP_DOUBLE:
			return val->v.d ;
		case MLP_STRING:
			return (val->v.s == NULL ? NAN : strtod(val->v.s, NULL)) ;
		default:
			return NAN ;
	}
}


extern "C" const char *mlp_value_to_string(mlp_value *val){
	if (val->stringy != NULL){
		return val->stringy ;
	}

	switch (val->type){
		case MLP_STRING:
			return val->v.s ;
		case MLP_INT: {
			char num[1024] ;
			snprintf(num, 1023, "%d", val->v.i) ;
			val->stringy = strdup(num) ;
			return val->stringy ;
		}
		case MLP_LONG: {
			char num[1024] ;
			snprintf(num, 1023, "%ld", val->v.l) ;
			val->stringy = strdup(num) ;
			return val->stringy ;
		}
		case MLP_DOUBLE: {
			char num[1024] ;
			snprintf(num, 1023, "%.15lf", val->v.d) ;
			val->stringy = strdup(num) ;
			return val->stringy ;
		}
		default:
			return NULL ;
	}
}


extern "C" const mlp_object *mlp_value_to_object(const mlp_value *val){
	switch (val->type){
		case MLP_INT:
		case MLP_LONG:
		case MLP_DOUBLE:
		case MLP_STRING:
			return NULL ;
		default:
			return val->v.o ;
	}
}


extern "C" mlp_type mlp_value_get_type(const mlp_value *val){
	return val->type ;
}
