#ifndef _MLP_IMPL_HPP_
#define _MLP_IMPL_HPP


#include "mlp.h"


/* MLP Language */
class mlp_language_impl {
	std::string name ;
	static std::map<std::string, mlp_language *> languages ;
	void *handle ;

	protected:
		mlp_language(const char *name) ;
		~mlp_language() ;
    private:
		void set_handle(void *handle) ;
    public:
	    static mlp_language *init(mlp_context *ctx, const char *name) ;
	    virtual mlp_runtime *new_runtime(mlp_context *ctx, const void *opts) = 0 ;
	    const char *get_name(mlp_context *ctx) ;
	    void destroy(mlp_context *ctx) ;


	private: 
		mlp_language() ;
		~mlp_language() ;
	public: 
		static mlp_language *init(mlp_context *ctx, const char *name) ;
		mlp_runtime *new_runtime(mlp_context *ctx, const void *opts) ; 
		void destroy(mlp_context *ctx) ;
} ;


/* MLP Runtime */
class mlp_runtime {
	mlp_language *lang ;

	protected:
	mlp_runtime(mlp_context *ctx, mlp_language *lang, const void *opts) ;
	virtual ~mlp_runtime() ;
	public:
	virtual mlp_library *load_library(mlp_context *ctx, const char *name, const void *opts) = 0 ;
	virtual void setup_context(mlp_context *ctx) = 0 ;
	//virtual value *call_function(context *ctx, const char *name, int nb_args, const value **args) = 0 ;
	//virtual object *new_object(context *ctx, const clazz *cls, int nb_args, const value **args) = 0 ;
	//virtual mlp_array *new_array(mlp_context *ctx, mlp_type type, mlp_arguments *args) = 0 ;
	mlp_language *get_language(mlp_context *ctx) ;
	virtual void destroy(mlp_context *ctx) ;
} ;


/* MLP Library */
class mlp_library {
	protected:
	std::string name ;
	mlp_runtime *rt ;

	protected:
	mlp_library(mlp_context *ctx, mlp_runtime *rt, const char *name, const void *opts) ;
	virtual ~mlp_library() ;
	public:
	virtual mlp_function *get_function(mlp_context *ctx, 
		mlp_type rtype, const char *name, mlp_prototype proto) = 0 ;
	const char *get_name(mlp_context *ctx) ;
	mlp_runtime *get_runtime(mlp_context *ctx) ;
	virtual void destroy(mlp_context *ctx) ;
} ;


/* MLP Function */
class mlp_function {
	protected:
	std::string name ;
	mlp_runtime *rt ;
	mlp_library *lib ;
	mlp_type rtype ;
	mlp_prototype proto ;

	protected:
	mlp_function(mlp_context *ctx, mlp_runtime *rt, mlp_library *lib, 
		mlp_type rtype, const char *name, mlp_prototype proto) ;
	virtual ~mlp_function() ;
	public:
	virtual mlp_value *call(mlp_context *ctx, mlp_arguments args) = 0 ;
	const char *get_name(mlp_context *ctx) ;
	mlp_runtime *get_runtime(mlp_context *ctx) ;
	mlp_library *get_library(mlp_context *ctx) ;
	mlp_type get_return_type(mlp_context *ctx) ;
	virtual void destroy(mlp_context *ctx) ;
} ;




/* MLP Array */
class mlp_array {
	mlp_type t ;
	int len ;

	protected:
	public:
	int get_length(mlp_context *ctx) ;
	mlp_value *get_element(mlp_context *ctx, int idx) ;
	void *set_element(mlp_context *ctx, int idx, mlp_value *v) ;
} ;


/* A class */
class mlp_class {
	char *name ;

	public:
	const char *get_name() ;
} ;


/* An object */
class mlp_object {
	mlp_class *cls ;

	public:
	mlp_class *get_class() ;
} ;


#else


#define MLP_EXTERN_C
struct _mlp_context ;
typedef struct _mlp_context mlp_context ;
struct _mlp_value ;
typedef struct _mlp_value mlp_value ;
struct _mlp_language ;
typedef struct _mlp_language mlp_language ;
struct _mlp_runtime ;
typedef struct _mlp_runtime mlp_runtime ;
struct _mlp_library ;
typedef struct _mlp_library mlp_library ;
struct _mlp_function ;
typedef struct _mlp_function mlp_function ;
struct _mlp_array ;
typedef struct _mlp_array mlp_array ;
struct _mlp_object ;
typedef struct _mlp_object mlp_object ;


#endif

#include "mlp_p.h"

#endif
