#include "test.h"

void t_00_info(const char *MLP_TEST_LANG, const char *MLP_TEST_OPTS){
#line 1 "00_info.t"
	
	plan_tests(1) ;
	
	ok(1, "Test language is %s (%s)", MLP_TEST_LANG, MLP_TEST_OPTS) ;
}

void t_01_context(const char *MLP_TEST_LANG, const char *MLP_TEST_OPTS){
#line 1 "01_context.t"
	plan_tests(10) ;
	
	mlp_context ctx ;
	ok(! ctx.has_error(), "Context doesn't have a pending error") ;
	ok(ctx.get_error() == MLP_OK, "Context error is MLP_OK") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.set_error(MLP_INTERNAL_ERROR, NULL) ;
	ok(ctx.get_error() == MLP_INTERNAL_ERROR, "Context error is MLP_INTERNAL_ERROR") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.clear_error() ;
	ok(ctx.get_error() == MLP_OK, "Context error is MLP_OK") ;
	ok(ctx.get_error_msg() == NULL, "Context error msg is NULL") ;
	
	ctx.set_error(MLP_INTERNAL_ERROR, "internal %s", "error") ;
	ok(ctx.get_error() == MLP_INTERNAL_ERROR, "Context error is MLP_INTERNAL_ERROR") ;
	ok(strcmp(ctx.get_error_msg(), "internal error") == 0, "Context error msg is \"internal error\"") ;
	
	mlp_runtime *rt = NULL ;
	ctx.set_runtime_data(rt, (const void *)"data") ;
	ok(strcmp((const char  *)ctx.get_runtime_data(rt), "data") == 0, "Context runtime data for NULL is \"data\"") ;
}

void t_02_value(const char *MLP_TEST_LANG, const char *MLP_TEST_OPTS){
#line 1 "02_value.t"
	plan_tests(18) ;
	
	{
	mlp_value i(42) ;
	ok(i.to_int() == 42, "Int value as long is 42") ;
	ok(i.to_double() == (double)42, "Int value as double is 42.0") ;
	ok(strcmp(i.to_string(), "42") == 0, "Int value as string is \"42\"") ;
	}
	
	{
	mlp_value l(42L) ;
	ok(l.to_long() == 42L, "Long value as long is 42") ;
	ok(l.to_double() == (double)42L, "Long value as double is 42.0") ;
	ok(strcmp(l.to_string(), "42") == 0, "Long value as string is \"42\"") ;
	}
	
	{
	mlp_value d(42.6) ;
	ok(d.to_double() == 42.6, "Double value as double is 42.6") ;
	ok(d.to_long() == 42L, "Double value as long is 42") ;
	ok(strncmp(d.to_string(), "42.6", 4) == 0, "Double value as string is \"42.6\"", d.to_string()) ;
	}
	
	{
	mlp_value s("42.6") ;
	ok(strcmp(s.to_string(), "42.6") == 0, "String value as string is \"42.6\"") ;
	ok(s.to_long() == 42L, "String value as long is 42") ;
	ok(s.to_double() == 42.6, "String value as double is 42.6") ;
	}
	
	
	{
	mlp_value s("some string") ;
	ok(strcmp(s.to_string(), "some string") == 0, "String value as string is \"some string\"") ;
	ok(s.to_long() == 0L, "String value as long is 0") ;
	ok(s.to_double() == (double)0L, "String value as double is 0.0") ;
	}
	
	
	{
	mlp_value s((const char *)NULL) ;
	ok(s.to_string() == NULL, "String value as string is NULL") ;
	ok(s.to_long() == 0L, "String value as long is 0") ;
	// Here we test NANness by using x != x
	ok(s.to_double() != s.to_double(), "String value as double is NAN") ;
	}
}

void t_03_language(const char *MLP_TEST_LANG, const char *MLP_TEST_OPTS){
#line 1 "03_language.t"
	plan_tests(2) ;
	
	mlp_context ctx ;
	
	mlp_language *lang = mlp_language::init(&ctx, MLP_TEST_LANG) ; // MLP_TEST_OPTS ;
	MLP_CHECK_ERROR(ctx, 1){
	ok(lang != NULL, "Language %s initialized", MLP_TEST_LANG) ;
	}
	
	
	lang->destroy(&ctx) ;
	MLP_CHECK_ERROR(ctx, 1){
	pass("Language %s destroyed", MLP_TEST_LANG) ;
	}
}

int main(int argc, char *argv[]){
	const char *l = getenv("MLP_TEST_LANG") ;
	const char *o = getenv("MLP_TEST_OPTS") ;
	if (argc < 2) { fprintf(stderr, "Usage: test FILE\n") ; exit(1) ; }
	if (strcmp(argv[argc-1], "00_info.t") == 0) { t_00_info(l, o) ; }
	if (strcmp(argv[argc-1], "01_context.t") == 0) { t_01_context(l, o) ; }
	if (strcmp(argv[argc-1], "02_value.t") == 0) { t_02_value(l, o) ; }
	if (strcmp(argv[argc-1], "03_language.t") == 0) { t_03_language(l, o) ; }
	return exit_status() ;
}
