package mlp.java ;


import java.util.* ;
import java.lang.reflect.* ;
import mlp.* ;


/*
 *  This class in the entry point though which all the access to Java is done.
 *  The interface is meant to be simple to use since it will be invoked primarily
 *  by the mlp_java C-language plugin.
 */
public class MLPJavaAPI {
	/* 
	 *  Creates a new object of class clazz, using args[] as the parameter list.
	 *  if sign[] is not null, use the class names in the array to help identify
	 *  proper constructor.
	 */ 
	public static MLPValue newObject(String clazz, MLPValue args[], String hint) throws MLPJavaException {
		MLPUtils.debug(1, "createObject: '" + clazz + "', '" + MLPJavaUtils.makeSignature(args) + "', '" + (hint != null ? hint : "") + "'") ; 
		Class c = MLPJavaUtils.find(clazz) ;
		Class proto[] = null ;
		try {
			if ((args == null)||(args.length == 0)){
				// Try to use the default constructor.
				proto = new Class [] {} ;
				Object o = c.newInstance() ;
				return new MLPValue(new MLPJavaObject(o, o.getClass())) ;
			}

			ArrayList<Constructor> candidates = new ArrayList<Constructor>() ;
			Constructor cons[] = c.getConstructors() ;
			for (int i = 0 ; i < cons.length ; i++){
				Constructor con = cons[i] ;
				Class params[] = con.getParameterTypes() ;
				if ((hint != null)&&(! hint.equals(""))&&(! hint.equals(MLPJavaUtils.makeSignature(params)))){
					// If we have a hint, it must match exactly.
					continue ; 
				}
				else if (MLPJavaUtils.canCast(params, args)){
					candidates.add(con) ;
				}
			}

			if (candidates.size() == 0){
                throw new MLPJavaException("Constructor for class " + clazz + " with signature matching " +
					(hint != null ? hint : MLPJavaUtils.makeSignature(args)) + " not found") ;
			}
			else if (candidates.size() > 1){
				StringBuffer choices = new StringBuffer() ;
				for (int i = 0 ; i < candidates.size() ; i++){
					choices.append("  " + MLPJavaUtils.makeSignature(candidates.get(i).getParameterTypes()) + "\n") ;
				}
				throw new MLPJavaException("Can't choose amongst the following constructors for class " + clazz + 
					" with signature matching " + MLPJavaUtils.makeSignature(args) + ":\n" + choices.toString()) ;
			}
			else {
				// Actually call the constructor...
				Constructor con = candidates.get(0) ;
				proto = con.getParameterTypes() ;
				Object o = con.newInstance(MLPJavaUtils.castArguments(proto, args)) ;
				return new MLPValue(new MLPJavaObject(o, o.getClass())) ; 
			}
		}
		catch (InstantiationException e){
			throw new MLPJavaException("You are not allowed to instantiate object of class " + 
				clazz + ": " + e.getMessage()) ;
		}
		catch (IllegalAccessException e){
			throw new MLPJavaException("You are not allowed to instantiate object of class " + 
				clazz + " using the constructor with signature " + 
				MLPJavaUtils.makeSignature(proto) + ": " + e.getMessage()) ;
        }
		catch (IllegalArgumentException e){
			throw new MLPJavaException("Arguments for constructor of class " + clazz + " with signature " +
				MLPJavaUtils.makeSignature(proto) + " are incompatible: " + e.getMessage()) ;
		}
		catch (InvocationTargetException e){
			Throwable cause = e.getCause() ;
			MLPValue o = new MLPValue(new MLPJavaObject(cause, cause.getClass())) ; 
			MLPValue ret = new MLPValue(new MLPJavaException(cause.getMessage(), o)) ; 
			return ret ;
        }
	}



	public static MLPValue callMethod(MLPJavaObject jobj, String method, MLPValue args[], String hint) throws MLPJavaException {
		Class c = MLPJavaUtils.find(jobj.getClassName()) ;
		MLPUtils.debug(1, "callMethod: '" + jobj.getObject() + "', '" + method + "', '" + MLPJavaUtils.makeSignature(args) + 
			"', '" + (hint != null ? hint : "") + "'") ; 
		Class proto[] = null ;
		try {
			ArrayList<Method> candidates = new ArrayList<Method>() ;
			Method meths[] = c.getMethods() ;
			for (int i = 0 ; i < meths.length ; i++){
				Method meth = meths[i] ;
				if (meth.getName().equals(method)){
					MLPUtils.debug(2, "method " + meth.getName() + " found") ;
					Class params[] = meth.getParameterTypes() ;
					if ((hint != null)&&(! hint.equals(""))&&(! hint.equals(MLPJavaUtils.makeSignature(params)))){
						// If we have a hint, it must match exactly.
						continue ; 
					}
					else if (MLPJavaUtils.canCast(params, args)){
						candidates.add(meth) ;
					}
				}
			}

			if (candidates.size() == 0){
                throw new MLPJavaException("Method " + method + " of class " + c.getName() + " with signature matching " +
					(hint != null ? hint : MLPJavaUtils.makeSignature(args)) + " not found") ;
			}
			else if (candidates.size() > 1){
				StringBuffer choices = new StringBuffer() ;
				for (int i = 0 ; i < candidates.size() ; i++){
					choices.append("  " + MLPJavaUtils.makeSignature(candidates.get(i).getParameterTypes()) + "\n") ;
				}
				throw new MLPJavaException("Can't choose amongst the following " + method + " methods of class " + 
					c.getName() + " with signature matching " + MLPJavaUtils.makeSignature(args) + ":\n" + 
					choices.toString()) ;
			}
			else {
				// Actually call the method...
				Method meth = candidates.get(0) ;
				proto = meth.getParameterTypes() ;
				Object o = meth.invoke(jobj.getObject(), MLPJavaUtils.castArguments(proto, args)) ;
				return MLPJavaUtils.wrapObject(o, meth.getReturnType()) ;
			}
		}
		catch (IllegalAccessException e){
			throw new MLPJavaException("You are not allowed to call method " + method + " of class " + 
				c.getName() + " with signature " + 
				MLPJavaUtils.makeSignature(proto) + ": " + e.getMessage()) ;
        }
		catch (IllegalArgumentException e){
			throw new MLPJavaException("Arguments for method " + method  + " of class " + c.getName() + " with signature " +
				MLPJavaUtils.makeSignature(proto) + " are incompatible: " + e.getMessage()) ;
		}
		catch (InvocationTargetException e){
			Throwable cause = e.getCause() ;
			MLPValue o = new MLPValue(new MLPJavaObject(cause, cause.getClass())) ; 
			MLPValue ret = new MLPValue(new MLPJavaException(cause.getMessage(), o)) ; 
			return ret ;
        }
	}
} ;
