package MLP::Perl::API ;
use strict ;
use MLP::Value ;
use MLP::Perl::Utils ;
use MLP::Perl::Object ;


sub new_object {
	my $pkg = shift ;
	my $class = shift ;
	my $args = shift ;
	my $hint = shift ;

	my @args = defined $args ? @{$args} : () ;
	MLP::Utils->debug(1, "new_object: '$class', '" . join(", ", map {$_->to_string()} @args) . ", $hint") ;
	my $st = MLP::Perl::Utils->find_class($class) ;
	my $ctor = $hint || "new" ;

	no strict 'refs' ;
	my $ctor_code = *{$st->{$hint || "new"}}{CODE} ;
	if (defined $ctor_code){
		my $ret = undef ;
		eval {
			$ret = $ctor_code->($class, map {$_->get_scalar()} @args) ;
		} ;
		if ($@){
			my $e = new MLP::Perl::Exception($@) ;
			return MLP::Value->new_exception($e) ;
		}
		else {
			my $o = new MLP::Perl::Object($ret) ;
			return MLP::Value->new_object($o) ;
		}
	}
	else {
		die("Constructor '$ctor' for class '$class' not found") ;
	}
}



1 ;

__DATA__
	public static MLPValue callMethod(MLPJavaObject jobj, String method, MLPValue args[], String hint) throws MLPJavaException {
		Class c = MLPJavaUtils.find(jobj.getClassName()) ;
		MLPUtils.debug(1, "callMethod: '" + jobj.getObject() + "', '" + method + "', '" + MLPJavaUtils.makeSignature(args) + 
			"', '" + (hint != null ? hint : "") + "'") ; 
		Class proto[] = null ;
		try {
			ArrayList<Method> candidates = new ArrayList<Method>() ;
			Method meths[] = c.getMethods() ;
			for (int i = 0 ; i < meths.length ; i++){
				Method meth = meths[i] ;
				if (meth.getName().equals(method)){
					MLPUtils.debug(2, "method " + meth.getName() + " found") ;
					Class params[] = meth.getParameterTypes() ;
					if ((hint != null)&&(! hint.equals(MLPJavaUtils.makeSignature(params)))){
						// If we have a hint, it must match exactly.
						continue ; 
					}
					else if (MLPJavaUtils.canCast(params, args)){
						candidates.add(meth) ;
					}
				}
			}

			if (candidates.size() == 0){
                throw new MLPJavaException("Method " + method + " of class " + c.getName() + " with signature matching " +
					(hint != null ? hint : MLPJavaUtils.makeSignature(args)) + " not found") ;
			}
			else if (candidates.size() > 1){
				StringBuffer choices = new StringBuffer() ;
				for (int i = 0 ; i < candidates.size() ; i++){
					choices.append("  " + MLPJavaUtils.makeSignature(candidates.get(i).getParameterTypes()) + "\n") ;
				}
				throw new MLPJavaException("Can't choose amongst the following " + method + " methods of class " + 
					c.getName() + " with signature matching " + MLPJavaUtils.makeSignature(args) + ":\n" + 
					choices.toString()) ;
			}
			else {
				// Actually call the method...
				Method meth = candidates.get(0) ;
				proto = meth.getParameterTypes() ;
				Object o = meth.invoke(jobj.getObject(), MLPJavaUtils.castArguments(proto, args)) ;
				return MLPJavaUtils.wrapObject(o, meth.getReturnType()) ;
			}
		}
		catch (IllegalAccessException e){
			throw new MLPJavaException("You are not allowed to call method " + method + " of class " + 
				c.getName() + " with signature " + 
				MLPJavaUtils.makeSignature(proto) + ": " + e.getMessage()) ;
        }
		catch (IllegalArgumentException e){
			throw new MLPJavaException("Arguments for method " + method  + " of class " + c.getName() + " with signature " +
				MLPJavaUtils.makeSignature(proto) + " are incompatible: " + e.getMessage()) ;
		}
		catch (InvocationTargetException e){
			Throwable cause = e.getCause() ;
			MLPValue o = new MLPValue(new MLPJavaObject(cause, cause.getClass())) ; 
			MLPValue ret = new MLPValue(new MLPJavaException(cause.getMessage(), o)) ; 
			return ret ;
        }
	}
} ;
