package MLP::Type ;
use strict ;
use constant ;
use Scalar::Util qw(blessed) ;


constant->import({
	BOOL => __PACKAGE__->new("BOOL", 0),
	LONG => __PACKAGE__->new("LONG", 1), 
	DOUBLE => __PACKAGE__->new("DOUBLE", 2), 
   	CHAR => __PACKAGE__->new("CHAR", 3), 
	STRING => __PACKAGE__->new("STRING", 4),
	OBJECT => __PACKAGE__->new("OBJECT", 5),
	EXCEPTION => __PACKAGE__->new("EXCEPTION", 6),
}) ;


sub new {
	my $pkg = shift ;
	my $name = shift ;
	my $no = shift ;

	my $this = {
		name => $name,
		no => $no,
	} ;

	return bless($this, $pkg) ;
}


sub get_name {
	my $this = shift ;
	return $this->{name} ;
}


sub get_no {
	my $this = shift ;
	return $this->{no} ;
}


sub looks_like {
	my $pkg = shift ;
	my $v = shift ;
	my $type = shift ;

	if ($type eq MLP::Type->BOOL){
		return 1 if (!ref $v) ;
	}
	elsif ($type eq MLP::Type->LONG){
		return 1 if (!ref $v) && ($v =~ /^[+-]?\d+$/) ;
	}
	elsif ($type eq MLP::Type->DOUBLE){
		return 1 if (!ref $v) && ($v =~ /^[+-]?(?:\d+(?:\.\d*)?|\.\d+)$/) ;
	}
	elsif ($type eq MLP::Type->CHAR){
		return 1 if (!ref $v) && (length($v) == 1) ;
	}
	elsif ($type eq MLP::Type->STRING){
		return 1 if !defined $v ;
		return 1 if !ref $v ;
	}
	elsif ($type eq MLP::Type->OBJECT){
		return 1 if !defined $v ;
		return 1 if blessed($v) && $v->isa("MLP::Object") ;
	}
	elsif ($type eq MLP::Type->EXCEPTION){
		return 1 if !defined $v ;
		return 1 if blessed($v) && $v->isa("MLP::Exception") ;
	}
	
	return 0 ;
}



1 ;
