#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "../src/mlp_priv.h"
#include "mlp_perl_priv.h"


void *_mlp_perl_load_library(mlp_context *ctx, const char *name, const void *opts){
	PerlInterpreter *my_perl = mlp_context_get_runtime(ctx)->runtime ;
	int len = strlen(name) ;
	if (opts != NULL){
		len += strlen((char *)opts) ;
	}

    char *buf = (char *)malloc((len + 32) * sizeof(char)) ;
	sprintf(buf, "use %s %s ;", name, (opts != NULL ? (char *)opts : "")) ;

    eval_pv(buf, FALSE) ;
    if (SvTRUE(ERRSV)){
        STRLEN n_a ;
		mlp_context_set_error(ctx, MLP_LOAD_LIBRARY_ERROR, "Can't load library %s: %s\n",
			name, SvPV(ERRSV, n_a)) ;
        free(buf) ;
        return NULL ;
    }

	mlp_debug(2, "Library %s loaded", name) ;
	if (buf != NULL){
		free(buf) ;
	}

	return (void *)my_perl ;
}


mlp_value *_mlp_perl_call_function(mlp_context *ctx, void *library, const char *name, 
	int nb_args, const mlp_value **args, mlp_type return_type){

	PerlInterpreter *my_perl = mlp_context_get_runtime(ctx)->runtime ;
	mlp_value *ret = NULL ;

	dSP ;
	ENTER ;
	SAVETMPS ;

	PUSHMARK(SP) ;
	int i = 0 ;
	for (i = 0 ; i < nb_args ; i++){
		SV *sv = mlp_perl_make_SV(ctx, args[i]) ;
		if (mlp_context_has_error(ctx)){
			return NULL ;
		}
		XPUSHs(sv) ;
	}
	PUTBACK ;

	int count = call_pv(name, G_SCALAR | G_EVAL) ;

	SPAGAIN ;

    if (SvTRUE(ERRSV)){
		/* TODO: Deal with the exception and return it to the caller */
        STRLEN n_a ;
		mlp_context_set_error(ctx, MLP_NO_SUCH_FUNCTION, "Error calling function %s: %s",
			name, SvPV(ERRSV, n_a)) ;
        return NULL ;
    }
	else if (count != 1){
		mlp_context_set_error(ctx, MLP_INVALID_RETURN_VALUE, "Perl function must return 1 value (not %d)", count) ;
        return NULL ;
	}

	STRLEN n_a ;
	/* TODO: Use return_type to return the correct type */
	ret = mlp_new_string_value(ctx, POPpx) ;
	mlp_debug_value(ret, "perl ret") ;

	FREETMPS ;
	LEAVE ;

	return ret ;
}
