#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#include "mlp_priv.h"


mlp_exception *mlp_exception_alloc(mlp_runtime* rt, const char *msg, mlp_value *value){
	mlp_exception *e = MLP_MALLOC(mlp_exception) ;
	e->runtime = rt ;
	e->message = strdup(msg) ;
	e->value = value ;
	e->stringy = NULL ;

	return e ;
}


const char *mlp_exception_get_message(const mlp_exception *e){
	return e->message ;
}


mlp_value *mlp_exception_get_value(const mlp_exception *e){
	return e->value ;
}


mlp_runtime *mlp_exception_get_runtime(const mlp_exception *e){
    return e->runtime ;
}


const char *mlp_exception_to_string(mlp_exception *e){
	if (e->stringy != NULL){
        return e->stringy ;
    }

	const char *valstr = (e->value != NULL ? mlp_value_to_string(e->value) : NULL) ;
	int len = strlen(e->message) + (valstr != NULL ? strlen(valstr) : 8) + 8 ;
	e->stringy = (char *)malloc(len * sizeof(char)) ;
	const char *d = (e->message != NULL ? "\"" : "") ;
	if (e->value != NULL){
		snprintf(e->stringy, len-1, "%s%s%s, %s", d, e->message, d, valstr) ;
	}
	else {
		snprintf(e->stringy, len-1, "%s%s%s, NULL", d, e->message, d) ;
	}

	return e->stringy ;
}


void mlp_exception_delete(mlp_context *ctx, mlp_exception *e){
	mlp_context_clear_error(ctx) ;

	free(e->message) ;
	if (e->stringy != NULL){
		free(e->stringy) ;
	}
	if (e->value != NULL){
		mlp_value_delete(ctx, e->value) ; 
	}
	free(e) ;
}
