package MLP::Value ;
use strict ;
use MLP::Type ;


sub _new {
	my $pkg = shift ;
	my $v = shift ;
	my $type = shift ;

	die("Can't create a MLP::Value of type " . $type->get_name() . " using argument '$v'")
		unless MLP::Type->looks_like($v, $type) ;

	my $this = {
		v => $v,
		type => $type,
	} ;

	return bless($this, $pkg) ;
}


sub new_bool {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new(($v ? 1 : 0), MLP::Type->BOOL) ;
}


sub get_bool {
	my $this = shift ;

	die("Can't call get_bool() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->BOOL ;

	return $this->{v} ;
}


sub new_long {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->LONG) ;
}


sub get_long {
	my $this = shift ;

	die("Can't call get_long() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->LONG ;

	return $this->{v} ;
}


sub new_double {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->DOUBLE) ;
}


sub get_double {
	my $this = shift ;

	die("Can't call get_double() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->DOUBLE ;

	return $this->{v} ;
}


sub new_char {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->CHAR) ;
}


sub get_char {
	my $this = shift ;

	die("Can't call get_char() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->CHAR ;

	return $this->{v} ;
}


sub new_string {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->STRING) ;
}


sub get_string {
	my $this = shift ;

	die("Can't call get_string() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->STRING ;

	return $this->{v} ;
}


sub new_object {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->OBJECT) ;
}


sub get_object {
	my $this = shift ;

	die("Can't call get_object() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->OBJECT ;

	return $this->{v} ;
}


sub new_exception {
	my $pkg = shift ;
	my $v = shift ;

	return $pkg->_new($v, MLP::Type->EXCEPTION) ;
}


sub get_exception {
	my $this = shift ;

	die("Can't call get_exception() on a MLP::Value of type " . $this->{type}->get_name()) 
		unless $this->{type} eq MLP::Type->EXCEPTION ;

	return $this->{v} ;
}


sub get_type {
	my $this = shift ;

	return $this->{type} ;
}


sub get_scalar {
	my $this = shift ;

	return $this->{v} ;
}


sub to_string {
	my $this = shift ;

	if ($this->{type} eq MLP::Type->OBJECT){
		return "OBJECT(NULL)" unless defined $this->{v} ;
		return "OBJECT(" . $this->{v}->to_string() . ")" ;
    }
	elsif ($this->{type} eq MLP::Type->EXCEPTION){
		return "EXCEPTION(NULL)" unless defined $this->{v} ;
		return "EXCEPTION(" . $this->{v}->to_string() . ")" ;
    }
	else {
		my $str = defined $this->{v} ? $this->{v} : "NULL" ;
		return $this->{type}->get_name() . "($str)" ;
	}
}

1 ;
