#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <dlfcn.h>
#include <stdarg.h>

#include "mlp.h"




mlp_interpreter *mlp_create_interpreter(char *name, mlp_context **ctx){
	/* The first thing we do is create the context */
	(*ctx) = (mlp_context *)malloc(sizeof(mlp_context)) ;
	(*ctx)->interp = NULL ;
	(*ctx)->error = NULL ;

	char *buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
	sprintf(buf, "libmlp-%s.so", name) ;
	void *handle = dlopen(buf, RTLD_LAZY) ;
	if (handle == NULL){
		mlp_set_error(*ctx, "Can't open shared object %s: %s\n", buf, dlerror()) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;
	
	/* Now that the plugin is loaded, we must initialize it */
	buf = (char *)malloc((strlen(name) + 64) * sizeof(char)) ;
	sprintf(buf, "_mlp_%s_init", name) ;

	dlerror() ; 
	mlp_interface *(*init)(mlp_context *ctx) = NULL ;
	*((void **)(&init)) = dlsym(handle, buf) ;
	char *error ;
	if ((error = dlerror()) != NULL){
		mlp_set_error(*ctx, "Can't find symbol %s: %s\n", buf, error) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;

	mlp_interface *iface = (*init)(*ctx) ;
	if (mlp_has_error(*ctx)){
		return NULL ;
	}

	void *plugin_interp = (*(iface->create_interpreter))(*ctx) ;
	if (mlp_has_error(*ctx)){
		return NULL ;
	}

	mlp_interpreter *interp = (mlp_interpreter *)malloc(sizeof(mlp_interpreter)) ;
	interp->name = strdup(name) ;
	interp->interface = iface ;
	interp->interpreter = plugin_interp ;
	interp->dl_handle = handle ;
	(*ctx)->interp = interp ;


	return interp ;
}


int mlp_load_module(mlp_context *ctx, const char *module){
	return 1 ;
}


mlp_object *mlp_create_object(mlp_context *ctx, const char *class, ...){
	va_list va ;
	va_start(va, class) ;
	mlp_interface *iface = ctx->interp->interface ;
	void *plugin_obj = (*(iface->create_object))(ctx, class, va) ;

	mlp_object *obj = (mlp_object *)malloc(sizeof(mlp_object)) ;
	obj->object = plugin_obj ;

	return obj ;
}


/*
mlp_value *mlp_call_function(mlp_context *ctx, mlp_type return_type, const char *pkg, const char *func, ...){
	va_list va ;
	va_start(va, class) ;
	mlp_interface *iface = ctx->interp->interface ;
	mlp_value *plugin_obj = (*(iface->call_function))(ctx, return_type, pkg, func, va) ;

	mlp_object *obj = (mlp_object *)malloc(sizeof(mlp_object)) ;
	obj->object = plugin_obj ;

	return obj ;
}
*/


mlp_value *mlp_call_function(mlp_context *ctx, mlp_type return_type, const char *pkg, const char *func, mlp_value *args){
	mlp_interface *iface = ctx->interp->interface ;
	fprintf(stderr, "%p %p %i %s %s %p\n", iface->call_function, ctx, return_type, pkg, func, args) ;
	mlp_value *value = (*(iface->call_function))(ctx, return_type, pkg, func, args) ;

	return value ;
}


/*
int mlp_destroy_interp(const mlp_interp *interp){
	mlp_ctx *ctx = mlp_create_ctx(interp) ;
	int (*impl_destroy_interp)(mlp_interp *interp) ;
	if ((impl_destroy_interp = get_symbol(ctx, interp->name, "destroy_interp")) == NULL){
		mlp_destroy_ctx(ctx) ;
		return 0 ;
	}
	else if (! ((*impl_destroy_interp)(interp, interp->ctx))){
		mlp_destroy_ctx(ctx) ;
		return 0 ;
	}
	mlp_destroy_ctx(ctx) ;

	free(interp->name) ;
	dlclose(interp->dl_handle) ;
	free(interp) ;

	return 1 ;
}


void *get_symbol(const mlp_context *ctx, const char *name, const char *symbol){
	char *buf = (char *)malloc((strlen(name) + 64) * sizeof(char)) ;
	sprintf(buf, "%s_%s", name, symbol) ;

	dlerror() ;   
	void *sym = dlsym(ctx->interp->dl_handle, buf) ;
	char *error ;
	if ((error = dlerror()) != NULL){
		mlp_set_error(ctx->error, "Can't find symbol %s: %s\n", buf, error) ;
		free(buf) ;
		return NULL ;
	}

	return sym ;
}


void mlp_create_ctx(const mlp_interp *interp, mlp_context **ctx){
	ctx = (mlp_context *)malloc(sizeof(mlp_context)) ;
	ctx->interp = interp ;
	ctx->error = NULL ;

	return 1 ;
}


void mlp_destroy_ctx(mlp_context *ctx){
	if (ctx->error != NULL){
		free(ctx->error) ;
	}
	free(ctx) ;
}
*/


