#ifndef _MLP_H_
#define _MLP_H_

#include <stdarg.h>


#define MLP_VERSION "0.01"


struct _mlp_interpreter ;
struct _mlp_context ;
struct _mlp_interface ;
struct _mlp_object ;
struct _mlp_value ;

typedef struct _mlp_interpreter mlp_interpreter ;
typedef struct _mlp_context mlp_context ;
typedef struct _mlp_interface mlp_interface ;

typedef enum _mlp_type mlp_type ;
typedef struct _mlp_value mlp_value ;
typedef struct _mlp_object mlp_object ;


struct _mlp_interpreter {
	char *name ;
	mlp_interface *interface ;
	void *interpreter ;
	void *dl_handle ;
	
} ;

struct _mlp_context {
	mlp_interpreter *interp ;
	void *context ;
	char *error ;
} ;


enum _mlp_type {
	MLP_LONG = 1,
	MLP_OBJECT
} ;

struct _mlp_value {
	mlp_type type ;
	union {
		long l ;
		int i ;
		mlp_object *o ;
	} value ;
} ;

struct _mlp_object {
	char *class ;
	void *object ;
} ;



struct _mlp_interface {
	void *(*create_interpreter)(mlp_context *ctx) ;
	void *(*create_object)(mlp_context *ctx, const char *class, va_list args) ;
	mlp_value *(*call_function)(mlp_context *ctx, mlp_type return_type, const char *pkg, const char *func, mlp_value *args) ;
} ;


mlp_interpreter *mlp_create_interpreter(char *name, mlp_context **ctx) ;
int mlp_load_module(mlp_context *ctx, const char *module) ;
mlp_value *mlp_call_function(mlp_context *ctx, mlp_type return_type, const char *pkg, const char *func, mlp_value *args) ;



#include "mlputils.h"

#endif

