#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include "mlp_priv.h"



mlp_context *mlp_new_context(){
	mlp_context *ctx = (mlp_context *)malloc(sizeof(mlp_context)) ;
	ctx->error = MLP_OK ;
	ctx->error_info = NULL ;
	ctx->runtime = NULL ;

	return ctx ;
}


mlp_error mlp_delete_context(mlp_context *ctx){
	if (ctx->error_info != NULL){
		free(ctx->error_info) ;
	}
	free(ctx) ;

	return MLP_OK ;
}


int mlp_context_has_error(const mlp_context *ctx){
	return (ctx->error == MLP_OK ? 0 : 1) ;
}


mlp_error mlp_context_get_error(const mlp_context *ctx){
	return ctx->error ;
}


char * mlp_context_get_error_info(const mlp_context *ctx){
	return ctx->error_info ;
}


mlp_error mlp_context_set_error(mlp_context *ctx, mlp_error err, const char *fmt, ...){
	mlp_context_clear_error(ctx) ;
	ctx->error = err ;

	if (fmt != NULL){
		va_list va ;
		va_start(va, fmt) ;

		char *info = (char *)malloc(MLP_MAX_ERROR_MSG_LENGTH * sizeof(char)) ;
		vsnprintf(info, MLP_MAX_ERROR_MSG_LENGTH - 1, fmt, va) ;
		va_end(va) ;
		ctx->error_info = info ;
	}

	return MLP_OK ;
}


mlp_error mlp_context_clear_error(mlp_context *ctx){
	ctx->error = MLP_OK ;
	if (ctx->error_info != NULL){
		free(ctx->error_info) ;
		ctx->error_info = NULL ;
	}

	return MLP_OK ;
}


mlp_error mlp_context_set_runtime(mlp_context *ctx, mlp_runtime *runtime){
	ctx->runtime = runtime ;

	return MLP_OK ;
}


mlp_runtime *mlp_context_get_runtime(const mlp_context *ctx){
	return ctx->runtime ;
}


