#ifndef _MLP_H_
#define _MLP_H_


#define MLP_VERSION "0.01"


enum _mlp_error ;
typedef enum _mlp_error mlp_error ;
enum _mlp_type ;
typedef enum _mlp_type mlp_type ;
struct _mlp_context ;
typedef struct _mlp_context mlp_context ;
struct _mlp_runtime ;
typedef struct _mlp_runtime mlp_runtime ;
struct _mlp_library ;
typedef struct _mlp_library mlp_library ;
struct _mlp_value ;
typedef struct _mlp_value mlp_value ;
struct _mlp_class ;
typedef struct _mlp_class mlp_class ;
struct _mlp_object ;
typedef struct _mlp_object mlp_object ;



/* Error codes */
enum _mlp_error {
	MLP_NOT_SUPPORTED = -1000,  /* Language does not support this feature */
	MLP_NOT_IMPLEMENTED= -1001, /* Language supports this feature but plugin doesn't implement it yet... */

	MLP_OK = 0,
	MLP_NEW_RUNTIME_ERROR,
	MLP_DL_ERROR,
	MLP_LOAD_LIBRARY_ERROR,
	MLP_NO_SUCH_FUNCTION,
	MLP_NO_SUCH_CLASS,
	MLP_INVALID_ARGUMENT,
	MLP_INVALID_RETURN_VALUE,
	MLP_INTERNAL_ERROR
} ;

/* Data types */
enum _mlp_type {
	/* MLP_UNKNOWN = 0, */
	MLP_LONG,
	MLP_DOUBLE,
	MLP_STRING,
} ;


/* Public API */
mlp_context *mlp_new_context() ;
int mlp_context_has_error(const mlp_context *ctx) ;
mlp_error mlp_context_get_error(const mlp_context *ctx) ;
mlp_error mlp_context_set_error(mlp_context *ctx, mlp_error error, const char *fmt, ...) ;
mlp_error mlp_context_clear_error(mlp_context *ctx) ;
mlp_error mlp_context_set_runtime(mlp_context *ctx, mlp_runtime *runtime) ;
mlp_runtime *mlp_context_get_runtime(const mlp_context *ctx) ;
mlp_error mlp_delete_context(mlp_context *ctx) ;

mlp_runtime *mlp_new_runtime(mlp_context *ctx, const char *name, const void *opts) ;
mlp_error mlp_delete_runtime(mlp_context *ctx, mlp_runtime *runtime) ;

/*
mlp_value *mlp_new_unknown_value(mlp_context *ctx, void *val) ;
void *mlp_get_unknown_value(mlp_context *ctx, const mlp_value *value) ;
*/
mlp_value *mlp_new_long_value(mlp_context *ctx, long val) ;
long mlp_get_long_value(mlp_context *ctx, const mlp_value *value) ;
mlp_value *mlp_new_double_value(mlp_context *ctx, double val) ;
double mlp_get_double_value(mlp_context *ctx, const mlp_value *value) ;
mlp_value *mlp_new_string_value(mlp_context *ctx, const char *val) ;
const char *mlp_get_string_value(mlp_context *ctx, const mlp_value *value) ;
mlp_value *mlp_convert_value(mlp_context *ctx, mlp_value *value, mlp_type type) ;
mlp_error mlp_delete_value(mlp_context *ctx, mlp_value *val) ;

mlp_library *mlp_load_library(mlp_context *ctx, const char *name, const void *opts) ;
mlp_value *mlp_call_function(mlp_context *ctx, const mlp_library *library, 
	const char *name, int nb_args, const mlp_value **args, mlp_type return_type) ;

mlp_class *mlp_new_class(mlp_context *ctx, const char *name) ;
mlp_object *mlp_new_object(mlp_context *ctx, const mlp_class *class_, 
	int nb_args, const mlp_value **args) ;
mlp_value *mlp_call_method(mlp_context *ctx, const mlp_object *object, 
	const char *name, int nb_args, const mlp_value **args, mlp_type return_type) ;
void mlp_detach_object(mlp_context *ctx, const mlp_object *object) ;
void mlp_delete_object(mlp_context *ctx, const mlp_object *object) ;

void mlp_debug(int level, const char *fmt, ...) ;
void mlp_fatal(const char *fmt, ...) ;



#endif

