#ifndef _MLP_PRIV_H_
#define _MLP_PRIV_H_


#include "../config.h"
#include "mlp.h"


#define MLP_MAX_ERROR_MSG_LENGTH	1024


struct _mlp_runtime_interface ;
typedef struct _mlp_runtime_interface mlp_runtime_interface ;


/* The MLP context */
struct _mlp_context {
    mlp_error error ;
    char *error_info ;
    mlp_runtime *runtime ;
} ;


/* A runtime */
struct _mlp_runtime {
    char *name ;
    mlp_runtime_interface *interface ;
    void *runtime ;
    void *dl_handle ;
} ;


/* A library */
struct _mlp_library {
    char *name ;
    void *library ;
} ;


/* A value can be passed as a function argument or returned by a function */
struct _mlp_value {
	mlp_type type ;
	union {
		void *v ;
		long l ;
		double d ;
		char *s ;
		mlp_object *o ;
	} value ;
} ;


/* A class */
struct _mlp_class {
    char *name ;
	void *class_ ;
} ;


/* An object */
struct _mlp_object {
    mlp_class *class_ ;
	void *object ;
} ;



struct _mlp_runtime_interface {
	void *(*new_runtime)(mlp_context *ctx, const void *opts) ;
	void (*delete_runtime)(mlp_context *ctx, void *runtime) ;

	void *(*load_library)(mlp_context *ctx, const char *name, const void *opts) ;

	mlp_value *(*convert_value)(mlp_context *ctx, mlp_value *value, mlp_type type) ;

	mlp_value *(*call_function)(mlp_context *ctx, void *library, const char *name, 
		int nb_args, const mlp_value **args, mlp_type return_type) ;

	void *(*new_class)(mlp_context *ctx, const char *name) ;
	void *(*new_object)(mlp_context *ctx, const mlp_class *class_,
	    int nb_args, const mlp_value **args) ;
	mlp_value *(*call_method)(mlp_context *ctx, const mlp_object *object, const char *name, 
		int nb_args, const mlp_value **args, mlp_type return_type) ;
	void (*detach_object)(mlp_context *ctx, const mlp_object *object) ;
	void (*delete_object)(mlp_context *ctx, const mlp_object *object) ;
} ;


#define MLP_GENERATE_INIT(iface, lang) \
	(mlp_runtime_interface *)malloc(sizeof(mlp_runtime_interface)) ; \
	/**/ \
	void *_mlp_ ## lang ## _new_runtime(mlp_context *ctx, const void *opts) ; \
	iface->new_runtime = _mlp_ ## lang ## _new_runtime ; \
	/**/ \
	void _mlp_ ## lang ## _delete_runtime(mlp_context *ctx, void *runtime) ; \
	iface->delete_runtime = _mlp_ ## lang ## _delete_runtime ; \
	/**/ \
	void *_mlp_ ## lang ## _load_library(mlp_context *ctx, const char *name, const void *opts) ; \
	iface->load_library = _mlp_ ## lang ## _load_library ; \
	/**/ \
	mlp_value *_mlp_## lang ## _convert_value(mlp_context *ctx, mlp_value *value, mlp_type type) ; \
	iface->convert_value = _mlp_ ## lang ## _convert_value ; \
	/**/ \
	mlp_value *_mlp_## lang ## _call_function(mlp_context *ctx, void *library, const char *name, \
		int nb_args, const mlp_value **args, mlp_type return_type) ; \
	iface->call_function = _mlp_ ## lang ## _call_function ; \
	/**/ \
	void *_mlp_## lang ## _new_class(mlp_context *ctx, const char *name) ; \
	iface->new_class = _mlp_ ## lang ## _new_class ; \
	void *_mlp_## lang ## _new_object(mlp_context *ctx, const mlp_class *class_, \
	    int nb_args, const mlp_value **args) ; \
	iface->new_object = _mlp_ ## lang ## _new_object ; \
	mlp_value *_mlp_## lang ## _call_method(mlp_context *ctx, const mlp_object *object, const char *name, \
		int nb_args, const mlp_value **args, mlp_type return_type) ; \
	iface->call_method = _mlp_ ## lang ## _call_method ; \
	void _mlp_## lang ## _detach_object(mlp_context *ctx, const mlp_object *object) ; \
	iface->detach_object = _mlp_ ## lang ## _detach_object ; \
	void _mlp_## lang ## _delete_object(mlp_context *ctx, const mlp_object *object) ; \
	iface->delete_object = _mlp_ ## lang ## _delete_object ; \

#endif

