#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <dlfcn.h>
#include <stdarg.h>

#include "mlp_priv.h"



mlp_runtime *mlp_new_runtime(mlp_context *ctx, const char *name, const void *opts){
	mlp_context_clear_error(ctx) ;
	dlerror() ; 
	char *buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
	sprintf(buf, "libmlp_%s.so", name) ;
	void *handle = dlopen(buf, RTLD_LAZY) ;
	if (handle == NULL){
		mlp_context_set_error(ctx, MLP_DL_ERROR, "Can't open shared object %s: %s\n", buf, dlerror()) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;

	/* Now that the plugin is loaded, we must initialize it */
	buf = (char *)malloc((strlen(name) + 32) * sizeof(char)) ;
	sprintf(buf, "_mlp_%s_init", name) ;

	dlerror() ; 
	mlp_runtime_interface *(*init)(const mlp_context *ctx) = NULL ;
	/* *((void **)(&init)) = dlsym(handle, buf) ; */
	init = dlsym(handle, buf) ;
	char *error ;
	if ((error = dlerror()) != NULL){
		mlp_context_set_error(ctx, MLP_DL_ERROR, "Can't find symbol %s: %s\n", buf, error) ;
		free(buf) ;
		return NULL ;
	}
	free(buf) ;

	mlp_runtime_interface *iface = (*init)(ctx) ;
	if (mlp_context_has_error(ctx)){
		return NULL ;
	}

	void *lang_runtime = (*(iface->new_runtime))(ctx, opts) ;
	if (mlp_context_has_error(ctx)){
		return NULL ;
	}

	mlp_runtime *runtime = (mlp_runtime *)malloc(sizeof(mlp_runtime)) ;
	runtime->name = strdup(name) ;
	runtime->interface = iface ;
	runtime->runtime = lang_runtime ;
	runtime->dl_handle = handle ;
	/* Finally associate the context with our newly created runtime */
	mlp_context_set_runtime(ctx, runtime) ;

	return runtime ;
}


mlp_error mlp_delete_runtime(mlp_context *ctx, mlp_runtime *runtime){
	mlp_context_clear_error(ctx) ;
    mlp_runtime_interface *iface = ctx->runtime->interface ;
    (*(iface->delete_runtime))(ctx, runtime->runtime) ;
	if (mlp_context_has_error(ctx)){
		return mlp_context_get_error(ctx) ;
	}

	dlclose(runtime->dl_handle) ;
	free(runtime->interface) ;
	free(runtime->name) ;
	free(runtime) ;

	return MLP_OK ;
}
