#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "mlp_priv.h"



static mlp_value *mlp_new_value(mlp_context *ctx, mlp_type type){
	mlp_context_clear_error(ctx) ;

	mlp_value *value = (mlp_value *)malloc(sizeof(mlp_value)) ;
	value->type = type ;

	return value ;
}


/*
mlp_value *mlp_new_unknown_value(mlp_context *ctx, void *val){
	mlp_value *value = mlp_new_value(ctx, MLP_UNKNOWN) ;
	value->value.v = val ;

	return value ;
}


void *mlp_get_unknown_value(mlp_context *ctx, const mlp_value *value){
	mlp_context_clear_error(ctx) ;
	return value->value.v ;
}
*/


mlp_value *mlp_new_long_value(mlp_context *ctx, long val){
	mlp_value *value = mlp_new_value(ctx, MLP_LONG) ;
	value->value.l = val ;

	return value ;
}


long mlp_get_long_value(mlp_context *ctx, const mlp_value *value){
	mlp_context_clear_error(ctx) ;
	return value->value.l ;
}


mlp_value *mlp_new_double_value(mlp_context *ctx, double val){
	mlp_value *value = mlp_new_value(ctx, MLP_DOUBLE) ;
	value->value.d = val ;

	return value ;
}


double mlp_get_double_value(mlp_context *ctx, const mlp_value *value){
	mlp_context_clear_error(ctx) ;
	return value->value.d ;
}


mlp_value *mlp_new_string_value(mlp_context *ctx, const char *val){
	mlp_value *value = mlp_new_value(ctx, MLP_STRING) ;
	value->value.s = (val == NULL ? NULL : strdup(val)) ;

	return value ;
}


const char *mlp_get_string_value(mlp_context *ctx, const mlp_value *value){
	mlp_context_clear_error(ctx) ;
	return value->value.s ;
}


mlp_value *mlp_convert_value(mlp_context *ctx, mlp_value *value, mlp_type type){
    mlp_context_clear_error(ctx) ;

	mlp_debug_value(value, "before convert") ;
	if (value == NULL){
		return value ;
	}

	mlp_type vtype = value->type ;
	if (vtype == type){
		return value ;
	}

	/* Handle some conversions between basic types */
	if (vtype == MLP_LONG){
		if (type == MLP_DOUBLE){
			value->type = MLP_DOUBLE ;
			value->value.d = (double)value->value.l ;
		}
		else if (type == MLP_STRING){
			char num[1024] ;
			value->type = MLP_STRING ;
			snprintf(num, 1023, "%ld", value->value.l) ;
			value->value.s = strdup(num) ;
		}
	}
	else if (vtype == MLP_DOUBLE){
		if (type == MLP_LONG){
			value->type = MLP_LONG ;
			value->value.l = (long)value->value.d ;
		}
		else if (type == MLP_STRING){
			char num[1024] ;
			value->type = MLP_STRING ;
			/* TODO: Set precision... */
			snprintf(num, 1023, "%lf", value->value.d) ;
			value->value.s = strdup(num) ;
		}
	}
	else if (vtype == MLP_STRING){
		if (type == MLP_LONG){
			value->type = MLP_LONG ;
			char *s = value->value.s ;
			if (s != NULL){
				value->value.l = strtol(s, NULL, 10) ;
				free(s) ;
			}
			else {
				value->value.l = 0L ;
			}
		}
		else if (type == MLP_DOUBLE){
			value->type = MLP_DOUBLE ;
			char *s = value->value.s ;
			if (s != NULL){
				value->value.d = strtod(s, NULL) ;
				free(s) ;
			}
			else {
				value->value.d = 0.0 ;
			}
		}
	}

	mlp_debug_value(value, "after convert") ;
	return value ;
		

	/*
    mlp_runtime_interface *iface = ctx->runtime->interface ;
    (*(iface->convert_value))(ctx, value, type) ;
    if (mlp_context_has_error(ctx)){
        return NULL ;
    }
	*/
}


mlp_error mlp_delete_value(mlp_context *ctx, mlp_value *value){
	mlp_context_clear_error(ctx) ;

	if (value != NULL){
		switch(value->type){
			case MLP_STRING: {
				char *s = value->value.s ;
				if (s != NULL){
					free(s) ;
				}
				break ;
			}
		} 
		free(value) ;
	}

    return MLP_OK ;
}
