
plan_tests(14) ;

mlp_context *ctx = mlp_new_context() ;
MLP_CHECK_ERROR(ctx, 1){
	ok(ctx != NULL, "Context initialized") ;
}

mlp_runtime *runtime = mlp_new_runtime(ctx, MLP_TEST_LANG, MLP_TEST_OPTS) ;
MLP_CHECK_ERROR(ctx, 1){
	ok(runtime != NULL, "Runtime initialized") ;
}

mlp_library *library = mlp_load_library(ctx, "libcalc", NULL) ;
MLP_CHECK_ERROR(ctx, 1){
	ok(library != NULL, "Library loaded") ;
}


mlp_value *args[2] ;
args[0] = mlp_new_long_value(ctx, 2) ;
args[1] = mlp_new_long_value(ctx, 3) ;
mlp_debug(5, "TEST: args initialized") ;
mlp_value *ret = mlp_call_function(ctx, library, "calc_add_long", 2, (const mlp_value **)args, MLP_LONG) ;
MLP_CHECK_ERROR(ctx, 2){
	ok(ret->type == MLP_LONG, "calc_add_long(2, 3) called and return value converted to long") ;
	long lsum = mlp_get_long_value(ctx, ret) ;
	MLP_CHECK_ERROR(ctx, 1){
		mlp_debug(5, "TEST: sum = %ld", lsum) ;
		ok(lsum == (2 + 3), "calc_add_long(2, 3) = 5") ;
	}
}

mlp_delete_value(ctx, args[0]) ;
mlp_delete_value(ctx, args[1]) ;
args[0] = mlp_new_double_value(ctx, 2.0) ;
args[1] = mlp_new_double_value(ctx, 3.0) ;
mlp_debug(5, "TEST: args initialized") ;
ret = mlp_call_function(ctx, library, "calc_add_double", 2, (const mlp_value **)args, MLP_DOUBLE) ;
MLP_CHECK_ERROR(ctx, 2){
	ok(ret->type == MLP_DOUBLE, "calc_add_double(2, 3) called and return value converted to double") ;
	double dsum = mlp_get_double_value(ctx, ret) ;
	MLP_CHECK_ERROR(ctx, 1){
		mlp_debug(5, "TEST: sum = %lf", dsum) ;
		ok(dsum == (2.0 + 3.0), "calc_add_double(2.0, 3.0) = 5.0") ;
	}
}

mlp_delete_value(ctx, args[0]) ;
mlp_delete_value(ctx, args[1]) ;
args[0] = mlp_new_string_value(ctx, "2.0") ;
args[1] = mlp_new_string_value(ctx, "3") ;
mlp_debug(5, "TEST: args initialized") ;
ret = mlp_call_function(ctx, library, "calc_add_string", 2, (const mlp_value **)args, MLP_STRING) ;
MLP_CHECK_ERROR(ctx, 2){
	ok(ret->type == MLP_STRING, "calc_add_string('2.0', '3') called and return value converted to string") ;
	const char *ssum = mlp_get_string_value(ctx, ret) ;
	MLP_CHECK_ERROR(ctx, 1){
		mlp_debug(5, "TEST: sum = %s", ssum) ;
		ok(!strncmp(ssum, "5.0", (strlen(ssum) < 3 ? strlen(ssum) : 3)), "calc_add_string('2.0', '3') = '5.0' (%s)", ssum) ;
	}
}



mlp_class *calc_class = mlp_new_class(ctx, "calc") ;
MLP_CHECK_ERROR(ctx, 5){
	ok(calc_class != NULL, "calc_class != NULL") ;

	mlp_object *calc_object = mlp_new_object(ctx, calc_class, 0, NULL) ;
	MLP_CHECK_ERROR(ctx, 3){
		ok(calc_object != NULL, "calc_object != NULL") ;
	
		mlp_delete_value(ctx, args[0]) ;
		mlp_delete_value(ctx, args[1]) ;
		args[0] = mlp_new_long_value(ctx, 2) ;
		args[1] = mlp_new_long_value(ctx, 3) ;
		mlp_debug(5, "TEST: args initialized") ;
		ret = mlp_call_method(ctx, calc_object, "add_long", 2, (const mlp_value **)args, MLP_LONG) ;
		MLP_CHECK_ERROR(ctx, 2){
			ok(ret->type == MLP_LONG, "calc_object->add_long(2, 3) called and return value converted to long") ;
			long lsum = mlp_get_long_value(ctx, ret) ;
			MLP_CHECK_ERROR(ctx, 1){
				mlp_debug(5, "TEST: sum = %ld", lsum) ;
				ok(lsum == (2 + 3), "calc_object->add_long(2, 3) = 5") ;
			}
		}
	}

	mlp_delete_object(ctx, calc_object) ;
	MLP_CHECK_ERROR(ctx, 1){
		ok(1, "calc_object deleted") ;
	}
}


mlp_delete_context(ctx) ;
